class FactorInfo:
    def __init__(self):
        self.Region = ""
        self.Pollutant = ""
        self.Source = ""
        self.Min = 0.0
        self.Max = 1.2
        self.Limit = 0.2
        self.BaseEmissionRatio = 0.0
        self.Index = 0